@@ -145,8 +145,8 @@ void check_missing_no_variance(double numeric_data[], size_t ncols, size_t nrows
             private(running_mean, mean_prev, running_ssq, cnt, col_stop, xval, min_val, max_val, min_decimals_col)
     for (size_t_for col = 0; col < ncols; col++) {
         running_mean = 0;
-        mean_prev = 0;
         running_ssq = 0;
+        mean_prev = numeric_data[col * nrows];
         min_val =  HUGE_VAL;
         max_val = -HUGE_VAL;
         cnt = 0;
@@ -178,11 +178,12 @@ void check_missing_no_variance(double numeric_data[], size_t ncols, size_t nrows
 void calc_central_mean_and_sd(size_t ix_arr[], size_t st, size_t end, double x[], size_t size_quarter, double *mean_central, double *sd_central)
 {
     long double running_mean = 0;
-    long double mean_prev    = 0;
     long double running_ssq  = 0;
+    long double mean_prev    = 0;
     double xval;
     size_t st_offset = st + size_quarter;
     if (ix_arr != NULL) {
+        mean_prev = x[ix_arr[st]];
         for (size_t row = st_offset; row <= (end - size_quarter); row++) {
             xval = x[ix_arr[row]];
             running_mean += (xval - running_mean) / (long double)(row - st_offset + 1);
@@ -190,6 +191,7 @@ void calc_central_mean_and_sd(size_t ix_arr[], size_t st, size_t end, double x[]
             mean_prev     = running_mean;
         }
     } else {
+        mean_prev = x[st_offset];
         for (size_t row = st_offset; row <= (end - size_quarter); row++) {
             xval = x[row];
             running_mean += (xval - running_mean) / (long double)(row - st_offset + 1);
@@ -684,6 +686,9 @@ void dealloc_ModelOutputs(ModelOutputs &model_outputs)
     model_outputs.~ModelOutputs();
 }
 
+bool interrupt_switch = false;
+bool handle_is_locked = false;
+
 /* Function to handle interrupt signals */
 void set_interrup_global_variable(int s)
 {
@@ -727,33 +732,41 @@ SignalSwitcher::SignalSwitcher()
 {
     #pragma omp critical
     {
-        interrupt_switch = false;
-        this->old_sig = signal(SIGINT, set_interrup_global_variable);
-        this->is_active = true;
+        if (!handle_is_locked)
+        {
+            handle_is_locked = true;
+            interrupt_switch = false;
+            this->old_sig = signal(SIGINT, set_interrup_global_variable);
+            this->is_active = true;
+        }
+
+        else {
+            this->is_active = false;
+        }
     }
 }
 
 SignalSwitcher::~SignalSwitcher()
 {
+    #ifndef _FOR_PYTHON
     #pragma omp critical
     {
-        if (this->is_active)
-            signal(SIGINT, this->old_sig);
-        this->is_active = false;
-        #ifndef _FOR_PYTHON
-        interrupt_switch = false;
-        #endif
+        if (this->is_active && handle_is_locked)
+            interrupt_switch = false;
     }
+    #endif
+    this->restore_handle();
 }
 
 void SignalSwitcher::restore_handle()
 {
     #pragma omp critical
     {
-        if (this->is_active)
+        if (this->is_active && handle_is_locked)
         {
             signal(SIGINT, this->old_sig);
             this->is_active = false;
+            handle_is_locked = false;
         }
     }
 }
